package ru.dbotthepony.mc.otm.capability;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.Collection;
import java.util.UUID;

public interface IMatterTaskProvider {
	/**
	 * @return immutable collection of tasks that can be allocated by a worker
	 */
	@Nonnull
	Collection<MatterTask> getTasks();

	/**
	 * @return immutable collection of all stored tasks, even fully allocated by workers
	 */
	@Nonnull
	Collection<MatterTask> getAllTasks();

	/**
	 * Allocates (marks as work-in-progress) a task
	 * by incrementing it's in_progress by 1
	 * and shrinking required by 1
	 *
	 * If required == 0, it should not be returned by this method
	 * @param simulate whenever to change internal state
	 * @return MatterTaskAllocation(task, pattern) that should be performed, or null if no work is available
	 */
	@Nullable
	MatterTaskAllocation allocateTask(boolean simulate);

	/**
	 * Notify about task completion. If this provider indeed contain this task, it should
	 * shrink in_progress by 1
	 * If in_progress == 0 and required == 0, it should discard the task
	 * @param task task being completed. this method should ignore tasks that are not owned by it.
	 * @return whenever task indeed belong to this provider and internal state was updated
	 */
	boolean notifyTaskCompletion(MatterTask task);

	/**
	 * @param id uuid of task
	 * @return MatterTask that this capability holds with this id, or null
	 */
	@Nullable
	MatterTask getTask(UUID id);

	/**
	 * Destroys all tasks this capability contains
	 */
	void dropAllTasks();
}
